{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "838cd433",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:54.775222Z",
     "iopub.status.busy": "2024-04-30T05:38:54.775056Z",
     "iopub.status.idle": "2024-04-30T05:38:55.135975Z",
     "shell.execute_reply": "2024-04-30T05:38:55.135396Z"
    }
   },
   "outputs": [],
   "source": [
    "import sympy as sm\n",
    "import sympy.physics.mechanics as me\n",
    "sm.init_printing(use_latex='mathjax')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "c0423b58",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.138582Z",
     "iopub.status.busy": "2024-04-30T05:38:55.138122Z",
     "iopub.status.idle": "2024-04-30T05:38:55.141941Z",
     "shell.execute_reply": "2024-04-30T05:38:55.141501Z"
    }
   },
   "outputs": [],
   "source": [
    "class ReferenceFrame(me.ReferenceFrame):\n",
    "\n",
    "    def __init__(self, *args, **kwargs):\n",
    "\n",
    "        kwargs.pop('latexs', None)\n",
    "\n",
    "        lab = args[0].lower()\n",
    "        tex = r'\\hat{{{}}}_{}'\n",
    "\n",
    "        super(ReferenceFrame, self).__init__(*args,\n",
    "                                             latexs=(tex.format(lab, 'x'),\n",
    "                                                     tex.format(lab, 'y'),\n",
    "                                                     tex.format(lab, 'z')),\n",
    "                                             **kwargs)\n",
    "me.ReferenceFrame = ReferenceFrame"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "295776de",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.143738Z",
     "iopub.status.busy": "2024-04-30T05:38:55.143572Z",
     "iopub.status.idle": "2024-04-30T05:38:55.146556Z",
     "shell.execute_reply": "2024-04-30T05:38:55.146077Z"
    }
   },
   "outputs": [],
   "source": [
    "N = me.ReferenceFrame('N')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "1720646a",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.148259Z",
     "iopub.status.busy": "2024-04-30T05:38:55.148098Z",
     "iopub.status.idle": "2024-04-30T05:38:55.150685Z",
     "shell.execute_reply": "2024-04-30T05:38:55.150241Z"
    }
   },
   "outputs": [],
   "source": [
    "a, b, c, d, e, f = sm.symbols('a, b, c, d, e, f')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "id": "09b93d82",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.152711Z",
     "iopub.status.busy": "2024-04-30T05:38:55.152359Z",
     "iopub.status.idle": "2024-04-30T05:38:55.157200Z",
     "shell.execute_reply": "2024-04-30T05:38:55.156645Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a\\hat{n}_x$"
      ],
      "text/plain": [
       "a n_x"
      ]
     },
     "execution_count": 5,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v = a*N.x\n",
    "v"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "64fa1de1",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.159221Z",
     "iopub.status.busy": "2024-04-30T05:38:55.158893Z",
     "iopub.status.idle": "2024-04-30T05:38:55.164029Z",
     "shell.execute_reply": "2024-04-30T05:38:55.163493Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}a\\\\0\\\\0\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡a⎤\n",
       "⎢ ⎥\n",
       "⎢0⎥\n",
       "⎢ ⎥\n",
       "⎣0⎦"
      ]
     },
     "execution_count": 6,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.to_matrix(N)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "ba17b26e",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.165907Z",
     "iopub.status.busy": "2024-04-30T05:38:55.165605Z",
     "iopub.status.idle": "2024-04-30T05:38:55.170077Z",
     "shell.execute_reply": "2024-04-30T05:38:55.169522Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a\\hat{n}_x + b\\hat{n}_y + c\\hat{n}_z$"
      ],
      "text/plain": [
       "a n_x + b n_y + c n_z"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "w = a*N.x + b*N.y + c*N.z\n",
    "w"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "9e6aa4df",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.171967Z",
     "iopub.status.busy": "2024-04-30T05:38:55.171643Z",
     "iopub.status.idle": "2024-04-30T05:38:55.177090Z",
     "shell.execute_reply": "2024-04-30T05:38:55.176624Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}a\\\\b\\\\c\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡a⎤\n",
       "⎢ ⎥\n",
       "⎢b⎥\n",
       "⎢ ⎥\n",
       "⎣c⎦"
      ]
     },
     "execution_count": 8,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "w.to_matrix(N)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "id": "2993b364",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.178967Z",
     "iopub.status.busy": "2024-04-30T05:38:55.178635Z",
     "iopub.status.idle": "2024-04-30T05:38:55.183252Z",
     "shell.execute_reply": "2024-04-30T05:38:55.182691Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle d\\hat{n}_x + e\\hat{n}_y + f\\hat{n}_z$"
      ],
      "text/plain": [
       "d n_x + e n_y + f n_z"
      ]
     },
     "execution_count": 9,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "x = d*N.x + e*N.y + f*N.z\n",
    "x"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "ac2708ff",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.185066Z",
     "iopub.status.busy": "2024-04-30T05:38:55.184904Z",
     "iopub.status.idle": "2024-04-30T05:38:55.189990Z",
     "shell.execute_reply": "2024-04-30T05:38:55.189518Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (a + d)\\hat{n}_x + (b + e)\\hat{n}_y + (c + f)\\hat{n}_z$"
      ],
      "text/plain": [
       "(a + d) n_x + (b + e) n_y + (c + f) n_z"
      ]
     },
     "execution_count": 10,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "w + x"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "id": "d44da3b6",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.191835Z",
     "iopub.status.busy": "2024-04-30T05:38:55.191553Z",
     "iopub.status.idle": "2024-04-30T05:38:55.196187Z",
     "shell.execute_reply": "2024-04-30T05:38:55.195731Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle 2 a\\hat{n}_x + 2 b\\hat{n}_y + 2 c\\hat{n}_z$"
      ],
      "text/plain": [
       "2⋅a n_x + 2⋅b n_y + 2⋅c n_z"
      ]
     },
     "execution_count": 11,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "y = 2*w\n",
    "y"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "id": "e9d3e24d",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.198267Z",
     "iopub.status.busy": "2024-04-30T05:38:55.197925Z",
     "iopub.status.idle": "2024-04-30T05:38:55.202551Z",
     "shell.execute_reply": "2024-04-30T05:38:55.202075Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle -  a\\hat{n}_x -  b\\hat{n}_y -  c\\hat{n}_z$"
      ],
      "text/plain": [
       "-a n_x + -b n_y + -c n_z"
      ]
     },
     "execution_count": 12,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "z = -w\n",
    "z"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "id": "7968f5d4",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.204522Z",
     "iopub.status.busy": "2024-04-30T05:38:55.204111Z",
     "iopub.status.idle": "2024-04-30T05:38:55.207135Z",
     "shell.execute_reply": "2024-04-30T05:38:55.206682Z"
    }
   },
   "outputs": [],
   "source": [
    "N = me.ReferenceFrame('N')\n",
    "l, theta = sm.symbols('l, theta')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "id": "8f1c69f0",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.209167Z",
     "iopub.status.busy": "2024-04-30T05:38:55.208752Z",
     "iopub.status.idle": "2024-04-30T05:38:55.217228Z",
     "shell.execute_reply": "2024-04-30T05:38:55.216691Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\frac{\\sqrt{2} l}{2}\\hat{n}_x + \\frac{\\sqrt{2} l}{2}\\hat{n}_y$"
      ],
      "text/plain": [
       "√2⋅l       √2⋅l\n",
       "──── n_x + ──── n_y\n",
       " 2          2"
      ]
     },
     "execution_count": 14,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v1 = l*sm.cos(sm.pi/4)*N.x + l*sm.sin(sm.pi/4)*N.y\n",
    "v1"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "id": "fe9970a2",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.219204Z",
     "iopub.status.busy": "2024-04-30T05:38:55.218802Z",
     "iopub.status.idle": "2024-04-30T05:38:55.222796Z",
     "shell.execute_reply": "2024-04-30T05:38:55.222241Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle - 10\\hat{n}_y$"
      ],
      "text/plain": [
       "-10 n_y"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v2 = -10*N.y\n",
    "v2"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "id": "27dfb4c2",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.224805Z",
     "iopub.status.busy": "2024-04-30T05:38:55.224442Z",
     "iopub.status.idle": "2024-04-30T05:38:55.230769Z",
     "shell.execute_reply": "2024-04-30T05:38:55.230284Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle -  l \\sin{\\left(\\theta \\right)}\\hat{n}_x + l \\cos{\\left(\\theta \\right)}\\hat{n}_y$"
      ],
      "text/plain": [
       "-l⋅sin(θ) n_x + l⋅cos(θ) n_y"
      ]
     },
     "execution_count": 16,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v3 = -l*sm.sin(theta)*N.x + l*sm.cos(theta)*N.y\n",
    "v3"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "id": "82aab1a7",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.232700Z",
     "iopub.status.busy": "2024-04-30T05:38:55.232337Z",
     "iopub.status.idle": "2024-04-30T05:38:55.239497Z",
     "shell.execute_reply": "2024-04-30T05:38:55.239019Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (5 l \\sin{\\left(\\theta \\right)} + \\frac{\\sqrt{2} l}{2})\\hat{n}_x + (- 5 l \\cos{\\left(\\theta \\right)} + \\frac{\\sqrt{2} l}{2} - 10)\\hat{n}_y$"
      ],
      "text/plain": [
       "⎛             √2⋅l⎞       ⎛              √2⋅l     ⎞\n",
       "⎜5⋅l⋅sin(θ) + ────⎟ n_x + ⎜-5⋅l⋅cos(θ) + ──── - 10⎟ n_y\n",
       "⎝              2  ⎠       ⎝               2       ⎠"
      ]
     },
     "execution_count": 17,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v1 + v2 - 5*v3"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "id": "ec93caed",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.241425Z",
     "iopub.status.busy": "2024-04-30T05:38:55.241082Z",
     "iopub.status.idle": "2024-04-30T05:38:55.245118Z",
     "shell.execute_reply": "2024-04-30T05:38:55.244641Z"
    }
   },
   "outputs": [],
   "source": [
    "N = me.ReferenceFrame('N')\n",
    "w = a*N.x + b*N.y + c*N.z\n",
    "x = d*N.x + e*N.y + f*N.z"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 19,
   "id": "638d6e7b",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.246856Z",
     "iopub.status.busy": "2024-04-30T05:38:55.246693Z",
     "iopub.status.idle": "2024-04-30T05:38:55.252332Z",
     "shell.execute_reply": "2024-04-30T05:38:55.251750Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a d + b e + c f$"
      ],
      "text/plain": [
       "a⋅d + b⋅e + c⋅f"
      ]
     },
     "execution_count": 19,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "me.dot(w, x)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 20,
   "id": "59cc981d",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.254417Z",
     "iopub.status.busy": "2024-04-30T05:38:55.253986Z",
     "iopub.status.idle": "2024-04-30T05:38:55.258813Z",
     "shell.execute_reply": "2024-04-30T05:38:55.258259Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a d + b e + c f$"
      ],
      "text/plain": [
       "a⋅d + b⋅e + c⋅f"
      ]
     },
     "execution_count": 20,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "w.dot(x)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 21,
   "id": "c05e5279",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.260824Z",
     "iopub.status.busy": "2024-04-30T05:38:55.260434Z",
     "iopub.status.idle": "2024-04-30T05:38:55.273613Z",
     "shell.execute_reply": "2024-04-30T05:38:55.273153Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\frac{a}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_x + \\frac{b}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_y + \\frac{c}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_z$"
      ],
      "text/plain": [
       "        a                       b                       c\n",
       "───────────────── n_x + ───────────────── n_y + ───────────────── n_z\n",
       "   ______________          ______________          ______________\n",
       "  ╱  2    2    2          ╱  2    2    2          ╱  2    2    2\n",
       "╲╱  a  + b  + c         ╲╱  a  + b  + c         ╲╱  a  + b  + c"
      ]
     },
     "execution_count": 21,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "w.normalize()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 22,
   "id": "98331f37",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.275477Z",
     "iopub.status.busy": "2024-04-30T05:38:55.275300Z",
     "iopub.status.idle": "2024-04-30T05:38:55.282823Z",
     "shell.execute_reply": "2024-04-30T05:38:55.282354Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\frac{a}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_x + \\frac{b}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_y + \\frac{c}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_z$"
      ],
      "text/plain": [
       "        a                       b                       c\n",
       "───────────────── n_x + ───────────────── n_y + ───────────────── n_z\n",
       "   ______________          ______________          ______________\n",
       "  ╱  2    2    2          ╱  2    2    2          ╱  2    2    2\n",
       "╲╱  a  + b  + c         ╲╱  a  + b  + c         ╲╱  a  + b  + c"
      ]
     },
     "execution_count": 22,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "def normalize(vector):\n",
    "    return vector/sm.sqrt(me.dot(vector, vector))\n",
    "\n",
    "normalize(w)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 23,
   "id": "37044b4c",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.284722Z",
     "iopub.status.busy": "2024-04-30T05:38:55.284521Z",
     "iopub.status.idle": "2024-04-30T05:38:55.289290Z",
     "shell.execute_reply": "2024-04-30T05:38:55.288803Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\sqrt{a^{2} + b^{2} + c^{2}}$"
      ],
      "text/plain": [
       "   ______________\n",
       "  ╱  2    2    2 \n",
       "╲╱  a  + b  + c  "
      ]
     },
     "execution_count": 23,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "w.magnitude()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 24,
   "id": "9a4f769c",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.291067Z",
     "iopub.status.busy": "2024-04-30T05:38:55.290903Z",
     "iopub.status.idle": "2024-04-30T05:38:55.297544Z",
     "shell.execute_reply": "2024-04-30T05:38:55.297060Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\frac{a}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_x + \\frac{b}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_y + \\frac{c}{\\sqrt{a^{2} + b^{2} + c^{2}}}\\hat{n}_z$"
      ],
      "text/plain": [
       "        a                       b                       c\n",
       "───────────────── n_x + ───────────────── n_y + ───────────────── n_z\n",
       "   ______________          ______________          ______________\n",
       "  ╱  2    2    2          ╱  2    2    2          ╱  2    2    2\n",
       "╲╱  a  + b  + c         ╲╱  a  + b  + c         ╲╱  a  + b  + c"
      ]
     },
     "execution_count": 24,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "w/w.magnitude()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 25,
   "id": "1a362b68",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.299326Z",
     "iopub.status.busy": "2024-04-30T05:38:55.299166Z",
     "iopub.status.idle": "2024-04-30T05:38:55.303375Z",
     "shell.execute_reply": "2024-04-30T05:38:55.302902Z"
    }
   },
   "outputs": [],
   "source": [
    "N = me.ReferenceFrame('N')\n",
    "v1 = a*N.x + b*N.y + a*N.z\n",
    "v2 = b*N.x + a*N.y + b*N.z"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 26,
   "id": "6ab4aa92",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.305211Z",
     "iopub.status.busy": "2024-04-30T05:38:55.305048Z",
     "iopub.status.idle": "2024-04-30T05:38:55.321143Z",
     "shell.execute_reply": "2024-04-30T05:38:55.320674Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\operatorname{acos}{\\left(\\frac{3 a b}{\\sqrt{a^{2} + 2 b^{2}} \\sqrt{2 a^{2} + b^{2}}} \\right)}$"
      ],
      "text/plain": [
       "    ⎛            3⋅a⋅b            ⎞\n",
       "acos⎜─────────────────────────────⎟\n",
       "    ⎜   ___________    ___________⎟\n",
       "    ⎜  ╱  2      2    ╱    2    2 ⎟\n",
       "    ⎝╲╱  a  + 2⋅b  ⋅╲╱  2⋅a  + b  ⎠"
      ]
     },
     "execution_count": 26,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "sm.acos(v1.dot(v2) / (v1.magnitude()*v2.magnitude()))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 27,
   "id": "e18f11f0",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.323193Z",
     "iopub.status.busy": "2024-04-30T05:38:55.322751Z",
     "iopub.status.idle": "2024-04-30T05:38:55.327968Z",
     "shell.execute_reply": "2024-04-30T05:38:55.327428Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a\\hat{n}_x + b\\hat{n}_y + c\\hat{n}_z$"
      ],
      "text/plain": [
       "a n_x + b n_y + c n_z"
      ]
     },
     "execution_count": 27,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "N = me.ReferenceFrame('N')\n",
    "w = a*N.x + b*N.y + c*N.z\n",
    "w"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 28,
   "id": "4a1d8472",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.329841Z",
     "iopub.status.busy": "2024-04-30T05:38:55.329680Z",
     "iopub.status.idle": "2024-04-30T05:38:55.334126Z",
     "shell.execute_reply": "2024-04-30T05:38:55.333657Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle d\\hat{n}_x + e\\hat{n}_y + f\\hat{n}_z$"
      ],
      "text/plain": [
       "d n_x + e n_y + f n_z"
      ]
     },
     "execution_count": 28,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "x = d*N.x + e*N.y + f*N.z\n",
    "x"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 29,
   "id": "49b4eeca",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.336030Z",
     "iopub.status.busy": "2024-04-30T05:38:55.335686Z",
     "iopub.status.idle": "2024-04-30T05:38:55.343909Z",
     "shell.execute_reply": "2024-04-30T05:38:55.343438Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (b f - c e)\\hat{n}_x + (- a f + c d)\\hat{n}_y + (a e - b d)\\hat{n}_z$"
      ],
      "text/plain": [
       "(b⋅f - c⋅e) n_x + (-a⋅f + c⋅d) n_y + (a⋅e - b⋅d) n_z"
      ]
     },
     "execution_count": 29,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "me.cross(w, x)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 30,
   "id": "9a0940f1",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.345690Z",
     "iopub.status.busy": "2024-04-30T05:38:55.345530Z",
     "iopub.status.idle": "2024-04-30T05:38:55.352889Z",
     "shell.execute_reply": "2024-04-30T05:38:55.352311Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (b f - c e)\\hat{n}_x + (- a f + c d)\\hat{n}_y + (a e - b d)\\hat{n}_z$"
      ],
      "text/plain": [
       "(b⋅f - c⋅e) n_x + (-a⋅f + c⋅d) n_y + (a⋅e - b⋅d) n_z"
      ]
     },
     "execution_count": 30,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "w.cross(x)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 31,
   "id": "9730edc7",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.354854Z",
     "iopub.status.busy": "2024-04-30T05:38:55.354532Z",
     "iopub.status.idle": "2024-04-30T05:38:55.362774Z",
     "shell.execute_reply": "2024-04-30T05:38:55.362208Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\sqrt{36077}$"
      ],
      "text/plain": [
       "√36077"
      ]
     },
     "execution_count": 31,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "N = me.ReferenceFrame('N')\n",
    "\n",
    "p1 = 23*N.x - 12* N.y\n",
    "p2 = 16*N.x + 2*N.y - 4*N.z\n",
    "p3 = N.x + 14*N.z\n",
    "\n",
    "me.cross(p2 - p1, p3 - p1).magnitude() / 2"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 32,
   "id": "bba711cd",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.364651Z",
     "iopub.status.busy": "2024-04-30T05:38:55.364450Z",
     "iopub.status.idle": "2024-04-30T05:38:55.369937Z",
     "shell.execute_reply": "2024-04-30T05:38:55.369467Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a\\hat{a}_x + b\\hat{n}_y$"
      ],
      "text/plain": [
       "a a_x + b n_y"
      ]
     },
     "execution_count": 32,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "N = me.ReferenceFrame('N')\n",
    "A = me.ReferenceFrame('A')\n",
    "\n",
    "a, b, theta = sm.symbols('a, b, theta')\n",
    "\n",
    "v = a*A.x + b*N.y\n",
    "v"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 33,
   "id": "53ee412a",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.371936Z",
     "iopub.status.busy": "2024-04-30T05:38:55.371608Z",
     "iopub.status.idle": "2024-04-30T05:38:55.376212Z",
     "shell.execute_reply": "2024-04-30T05:38:55.375747Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle 2 a\\hat{a}_x + 2 b\\hat{n}_y$"
      ],
      "text/plain": [
       "2⋅a a_x + 2⋅b n_y"
      ]
     },
     "execution_count": 33,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v + v"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 34,
   "id": "daa07b6a",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.378355Z",
     "iopub.status.busy": "2024-04-30T05:38:55.377906Z",
     "iopub.status.idle": "2024-04-30T05:38:55.386478Z",
     "shell.execute_reply": "2024-04-30T05:38:55.386011Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a \\cos{\\left(\\theta \\right)}\\hat{n}_x + (a \\sin{\\left(\\theta \\right)} + b)\\hat{n}_y$"
      ],
      "text/plain": [
       "a⋅cos(θ) n_x + (a⋅sin(θ) + b) n_y"
      ]
     },
     "execution_count": 34,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "A.orient_axis(N, theta, N.z)\n",
    "\n",
    "v.express(N)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 35,
   "id": "90c00ca5",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.388347Z",
     "iopub.status.busy": "2024-04-30T05:38:55.388171Z",
     "iopub.status.idle": "2024-04-30T05:38:55.393345Z",
     "shell.execute_reply": "2024-04-30T05:38:55.392874Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (a + b \\sin{\\left(\\theta \\right)})\\hat{a}_x + b \\cos{\\left(\\theta \\right)}\\hat{a}_y$"
      ],
      "text/plain": [
       "(a + b⋅sin(θ)) a_x + b⋅cos(θ) a_y"
      ]
     },
     "execution_count": 35,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.express(A)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 36,
   "id": "819a5807",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.395198Z",
     "iopub.status.busy": "2024-04-30T05:38:55.395024Z",
     "iopub.status.idle": "2024-04-30T05:38:55.399386Z",
     "shell.execute_reply": "2024-04-30T05:38:55.398915Z"
    }
   },
   "outputs": [],
   "source": [
    "q1, q2, q3, q4, q5 = sm.symbols('q1, q2, q3, q4, q5')\n",
    "l1, l2, l3, l4 = sm.symbols('l1, l2, l3, l4')\n",
    "N = me.ReferenceFrame('N')\n",
    "A = me.ReferenceFrame('A')\n",
    "B = me.ReferenceFrame('B')\n",
    "C = me.ReferenceFrame('C')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 37,
   "id": "d5177fe6",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.401155Z",
     "iopub.status.busy": "2024-04-30T05:38:55.400981Z",
     "iopub.status.idle": "2024-04-30T05:38:55.419445Z",
     "shell.execute_reply": "2024-04-30T05:38:55.418917Z"
    }
   },
   "outputs": [],
   "source": [
    "A.orient_body_fixed(N, (q1, q2, 0), 'ZXZ')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 38,
   "id": "0417afba",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.421481Z",
     "iopub.status.busy": "2024-04-30T05:38:55.421158Z",
     "iopub.status.idle": "2024-04-30T05:38:55.425912Z",
     "shell.execute_reply": "2024-04-30T05:38:55.425338Z"
    }
   },
   "outputs": [],
   "source": [
    "B.orient_axis(A, q3, A.x)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 39,
   "id": "0dcc6595",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.428019Z",
     "iopub.status.busy": "2024-04-30T05:38:55.427520Z",
     "iopub.status.idle": "2024-04-30T05:38:55.446103Z",
     "shell.execute_reply": "2024-04-30T05:38:55.445511Z"
    }
   },
   "outputs": [],
   "source": [
    "C.orient_body_fixed(B, (q4, q5, 0), 'XZX')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 40,
   "id": "e4e16cb4",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.448320Z",
     "iopub.status.busy": "2024-04-30T05:38:55.447908Z",
     "iopub.status.idle": "2024-04-30T05:38:55.451595Z",
     "shell.execute_reply": "2024-04-30T05:38:55.451031Z"
    }
   },
   "outputs": [],
   "source": [
    "R_P1_P2 = l1*A.z\n",
    "R_P2_P3 = l2*B.z\n",
    "R_P3_P4 = l3*C.z - l4*C.y"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 41,
   "id": "def56f38",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.453651Z",
     "iopub.status.busy": "2024-04-30T05:38:55.453327Z",
     "iopub.status.idle": "2024-04-30T05:38:55.457813Z",
     "shell.execute_reply": "2024-04-30T05:38:55.457257Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{1}\\hat{a}_z + l_{2}\\hat{b}_z -  l_{4}\\hat{c}_y + l_{3}\\hat{c}_z$"
      ],
      "text/plain": [
       "l₁ a_z + l₂ b_z + -l₄ c_y + l₃ c_z"
      ]
     },
     "execution_count": 41,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "R_P1_P4 = R_P1_P2 + R_P2_P3 + R_P3_P4\n",
    "R_P1_P4"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 42,
   "id": "884cf4ea",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.459885Z",
     "iopub.status.busy": "2024-04-30T05:38:55.459567Z",
     "iopub.status.idle": "2024-04-30T05:38:55.533765Z",
     "shell.execute_reply": "2024-04-30T05:38:55.533210Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (l_{1} \\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{2} \\right)} + l_{2} \\left(\\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)} + \\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{2} \\right)}\\right) + l_{3} \\left(- \\left(\\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{2} \\right)} \\sin{\\left(q_{3} \\right)} - \\sin{\\left(q_{1} \\right)} \\cos{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)}\\right) \\sin{\\left(q_{4} \\right)} + \\left(\\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)} + \\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{2} \\right)}\\right) \\cos{\\left(q_{4} \\right)}\\right) - l_{4} \\left(\\left(\\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{2} \\right)} \\sin{\\left(q_{3} \\right)} - \\sin{\\left(q_{1} \\right)} \\cos{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)}\\right) \\cos{\\left(q_{4} \\right)} \\cos{\\left(q_{5} \\right)} + \\left(\\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)} + \\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{2} \\right)}\\right) \\sin{\\left(q_{4} \\right)} \\cos{\\left(q_{5} \\right)} - \\sin{\\left(q_{5} \\right)} \\cos{\\left(q_{1} \\right)}\\right))\\hat{n}_x + (- l_{1} \\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{1} \\right)} + l_{2} \\left(- \\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{1} \\right)} \\cos{\\left(q_{3} \\right)} - \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{1} \\right)} \\cos{\\left(q_{2} \\right)}\\right) + l_{3} \\left(- \\left(- \\sin{\\left(q_{2} \\right)} \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{1} \\right)} + \\cos{\\left(q_{1} \\right)} \\cos{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)}\\right) \\sin{\\left(q_{4} \\right)} + \\left(- \\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{1} \\right)} \\cos{\\left(q_{3} \\right)} - \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{1} \\right)} \\cos{\\left(q_{2} \\right)}\\right) \\cos{\\left(q_{4} \\right)}\\right) - l_{4} \\left(\\left(- \\sin{\\left(q_{2} \\right)} \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{1} \\right)} + \\cos{\\left(q_{1} \\right)} \\cos{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)}\\right) \\cos{\\left(q_{4} \\right)} \\cos{\\left(q_{5} \\right)} + \\left(- \\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{1} \\right)} \\cos{\\left(q_{3} \\right)} - \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{1} \\right)} \\cos{\\left(q_{2} \\right)}\\right) \\sin{\\left(q_{4} \\right)} \\cos{\\left(q_{5} \\right)} - \\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{5} \\right)}\\right))\\hat{n}_y + (l_{1} \\cos{\\left(q_{2} \\right)} + l_{2} \\left(- \\sin{\\left(q_{2} \\right)} \\sin{\\left(q_{3} \\right)} + \\cos{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)}\\right) + l_{3} \\left(\\left(- \\sin{\\left(q_{2} \\right)} \\sin{\\left(q_{3} \\right)} + \\cos{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)}\\right) \\cos{\\left(q_{4} \\right)} - \\left(\\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)} + \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{2} \\right)}\\right) \\sin{\\left(q_{4} \\right)}\\right) - l_{4} \\left(\\left(- \\sin{\\left(q_{2} \\right)} \\sin{\\left(q_{3} \\right)} + \\cos{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)}\\right) \\sin{\\left(q_{4} \\right)} \\cos{\\left(q_{5} \\right)} + \\left(\\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{3} \\right)} + \\sin{\\left(q_{3} \\right)} \\cos{\\left(q_{2} \\right)}\\right) \\cos{\\left(q_{4} \\right)} \\cos{\\left(q_{5} \\right)}\\right))\\hat{n}_z$"
      ],
      "text/plain": [
       "(l₁⋅sin(q₁)⋅sin(q₂) + l₂⋅(sin(q₁)⋅sin(q₂)⋅cos(q₃) + sin(q₁)⋅sin(q₃)⋅cos(q₂)) +\n",
       " l₃⋅(-(sin(q₁)⋅sin(q₂)⋅sin(q₃) - sin(q₁)⋅cos(q₂)⋅cos(q₃))⋅sin(q₄) + (sin(q₁)⋅s\n",
       "in(q₂)⋅cos(q₃) + sin(q₁)⋅sin(q₃)⋅cos(q₂))⋅cos(q₄)) - l₄⋅((sin(q₁)⋅sin(q₂)⋅sin(\n",
       "q₃) - sin(q₁)⋅cos(q₂)⋅cos(q₃))⋅cos(q₄)⋅cos(q₅) + (sin(q₁)⋅sin(q₂)⋅cos(q₃) + si\n",
       "n(q₁)⋅sin(q₃)⋅cos(q₂))⋅sin(q₄)⋅cos(q₅) - sin(q₅)⋅cos(q₁))) n_x + (-l₁⋅sin(q₂)⋅\n",
       "cos(q₁) + l₂⋅(-sin(q₂)⋅cos(q₁)⋅cos(q₃) - sin(q₃)⋅cos(q₁)⋅cos(q₂)) + l₃⋅(-(-sin\n",
       "(q₂)⋅sin(q₃)⋅cos(q₁) + cos(q₁)⋅cos(q₂)⋅cos(q₃))⋅sin(q₄) + (-sin(q₂)⋅cos(q₁)⋅co\n",
       "s(q₃) - sin(q₃)⋅cos(q₁)⋅cos(q₂))⋅cos(q₄)) - l₄⋅((-sin(q₂)⋅sin(q₃)⋅cos(q₁) + co\n",
       "s(q₁)⋅cos(q₂)⋅cos(q₃))⋅cos(q₄)⋅cos(q₅) + (-sin(q₂)⋅cos(q₁)⋅cos(q₃) - sin(q₃)⋅c\n",
       "os(q₁)⋅cos(q₂))⋅sin(q₄)⋅cos(q₅) - sin(q₁)⋅sin(q₅))) n_y + (l₁⋅cos(q₂) + l₂⋅(-s\n",
       "in(q₂)⋅sin(q₃) + cos(q₂)⋅cos(q₃)) + l₃⋅((-sin(q₂)⋅sin(q₃) + cos(q₂)⋅cos(q₃))⋅c\n",
       "os(q₄) - (sin(q₂)⋅cos(q₃) + sin(q₃)⋅cos(q₂))⋅sin(q₄)) - l₄⋅((-sin(q₂)⋅sin(q₃)\n",
       "+ cos(q₂)⋅cos(q₃))⋅sin(q₄)⋅cos(q₅) + (sin(q₂)⋅cos(q₃) + sin(q₃)⋅cos(q₂))⋅cos(q\n",
       "₄)⋅cos(q₅))) n_z"
      ]
     },
     "execution_count": 42,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "R_P1_P4.express(N)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 43,
   "id": "c9d25211",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.535823Z",
     "iopub.status.busy": "2024-04-30T05:38:55.535483Z",
     "iopub.status.idle": "2024-04-30T05:38:55.540823Z",
     "shell.execute_reply": "2024-04-30T05:38:55.540232Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{1} \\sin{\\left(q_{1} \\right)} \\sin{\\left(q_{2} \\right)}\\hat{n}_x -  l_{1} \\sin{\\left(q_{2} \\right)} \\cos{\\left(q_{1} \\right)}\\hat{n}_y + l_{1} \\cos{\\left(q_{2} \\right)}\\hat{n}_z$"
      ],
      "text/plain": [
       "l₁⋅sin(q₁)⋅sin(q₂) n_x + -l₁⋅sin(q₂)⋅cos(q₁) n_y + l₁⋅cos(q₂) n_z"
      ]
     },
     "execution_count": 43,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "R_P1_P2.express(N)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 44,
   "id": "15c44368",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.542632Z",
     "iopub.status.busy": "2024-04-30T05:38:55.542463Z",
     "iopub.status.idle": "2024-04-30T05:38:55.547270Z",
     "shell.execute_reply": "2024-04-30T05:38:55.546726Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left\\{l_{1}, q_{1}, q_{2}\\right\\}$"
      ],
      "text/plain": [
       "{l₁, q₁, q₂}"
      ]
     },
     "execution_count": 44,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "R_P1_P2.free_symbols(N)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 45,
   "id": "d3a5ce95",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.549427Z",
     "iopub.status.busy": "2024-04-30T05:38:55.549017Z",
     "iopub.status.idle": "2024-04-30T05:38:55.553477Z",
     "shell.execute_reply": "2024-04-30T05:38:55.552925Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left\\{l_{1}\\right\\}$"
      ],
      "text/plain": [
       "{l₁}"
      ]
     },
     "execution_count": 45,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "R_P1_P2.free_symbols(A)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 46,
   "id": "767c775f",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-04-30T05:38:55.555729Z",
     "iopub.status.busy": "2024-04-30T05:38:55.555219Z",
     "iopub.status.idle": "2024-04-30T05:38:55.561460Z",
     "shell.execute_reply": "2024-04-30T05:38:55.560836Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left\\{l_{1}, l_{2}, l_{3}, l_{4}, q_{1}, q_{2}, q_{3}, q_{4}, q_{5}\\right\\}$"
      ],
      "text/plain": [
       "{l₁, l₂, l₃, l₄, q₁, q₂, q₃, q₄, q₅}"
      ]
     },
     "execution_count": 46,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "R_P1_P4.free_symbols(N)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.14"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}